﻿
using System;
using System.Net.Http;
using System.Web.Http;
using System.Web.Security;
using AngularAspNetMvc.Models.Mappings;
using AngularAspNetMvc.Web.App_Start;
using Newtonsoft.Json;
using StructureMap;
using AngularAspNetMvc.Web;

namespace AngularAspNetMvc.Tests.Core
{
    public class ApiTest
    {
        private static bool _isSetup;
        private static readonly object Lock = new object();
        protected IContainer Container { get; private set; }
        protected ApiTest()
        {
            if (!_isSetup)
            {
                lock (Lock)
                {
                    if (!_isSetup)
                    {
                        StructuremapMvc.Start();
                        MappingConfiguration.CreateMaps();
                        _isSetup = true;
                    }
                }
            }
            Container = ObjectFactory.Container;
        }


        protected T GetInstance<T>()
        {
            return Container.GetInstance<T>();
        }

        protected HttpMessageInvoker PrepareServer()
        {

            var server = new HttpServer(CreateHttpConfiguration());
            return new HttpMessageInvoker(server);
        }

        protected HttpConfiguration CreateHttpConfiguration()
        {
            var config = new HttpConfiguration();
            WebApiConfig.Register(config);
            config.IncludeErrorDetailPolicy = IncludeErrorDetailPolicy.Always;
            config.DependencyResolver = new TestStructureMapResolver(ObjectFactory.Container);
            return config;
        }

        protected void AddAuthorizationCookie(string userId, object userData, HttpRequestMessage message)
        {

            var ticket = new FormsAuthenticationTicket(
                    1,
                    userId,
                    DateTime.UtcNow,
                    DateTime.UtcNow.AddHours(1),
                    false,
                    JsonConvert.SerializeObject(userData));

            string encryptedTicket = FormsAuthentication.Encrypt(ticket);

            message.Headers.Add("Cookie", new[]
            {
                FormsAuthentication.FormsCookieName + "=" + encryptedTicket + "; path=" + FormsAuthentication.FormsCookiePath
            });
        }

    }
}
