module app.contacts {

    import IUtilities = app.core.services.IUtilities;
    import IHttp = app.core.services.http.IHttp;
    import IHttpResult = app.core.services.http.IHttpResult;
    import IContactsService = app.contacts.services.IContactsService;
    import IContactInfo = app.contacts.models.IContactInfo;
    import IContact = app.contacts.models.IContact;
    import IContactsTypeService = app.contactTypes.services.IContactsTypeService;
    import IContactType = app.contactTypes.models.IContactType;

    interface IContactTypesEditScope extends app.core.controllers.ICoreScope {
        save: () => void;
        cancel: () => void;
        model: IContact;
        contactTypes: IContactType[];
    }

    interface IContactRouteParams extends ng.route.IRouteParamsService {
        id: any;
    }

    class ContactsEditController extends app.core.controllers.CoreController {

        constructor(
            private utilities: IUtilities,
            private $scope: IContactTypesEditScope,
            $routeParams: IContactRouteParams,
            private $location: ng.ILocationService,
            contactsService: IContactsService,
            contactsTypeService: IContactsTypeService) {
            super($scope);

            $scope.save = () => {
                contactsService.saveContact($scope.model, (result: IContact) => {
                    $location.path("/contacts");
                });
            };

            $scope.cancel = () => {
                $location.path("/contacts");
            };

            var id = $routeParams.id;
            if (id === "add") {
                contactsTypeService.getContactTypes((typesResult: IContactType[]) => {
                    $scope.contactTypes = typesResult;
                    if (typesResult) {
                        $scope.model = {
                            ContactId: 0,
                            ContactTypeId: 0,
                            FirstName: "",
                            LastName: "",
                            IsActive: true
                        };
                    }
                });
            } else {
                contactsTypeService.getContactTypes((typesResult: IContactType[]) => {
                    $scope.contactTypes = typesResult;
                    if (typesResult) {
                        contactsService.getContact(id, (result: IContact) => {
                            $scope.model = result;
                        });
                    }
                });
            }

        }
    }

    angular.module("app.contacts.contactsEditController",
        ["app.core.services.utilities", "app.contacts.services", "ngRoute", "app.contactTypes.services"])
        .controller("app.contacts.contactsEditController",
        ["utilities", "$scope", "$routeParams", "$location", "contactsService", "contactsTypeService",
            function (
                utilities: IUtilities,
                $scope: IContactTypesEditScope,
                $routeParams: IContactRouteParams,
                $location: ng.ILocationService,
                contactsService: IContactsService,
                contactsTypeService: IContactsTypeService) {
                return new ContactsEditController(utilities, $scope, $routeParams, $location, contactsService, contactsTypeService);
            }]);
}