

module app.contacts.services {

    import IHttpResult = app.core.services.http.IHttpResult;
    import IContact = app.contacts.models.IContact;
    import IContactInfo = app.contacts.models.IContactInfo;
    import IPagedCriteria = app.common.models.IPagedCriteria;
    import IPagedResult = app.core.services.http.IPagedResult;
    import IHttpPagedResult = app.core.services.http.IPagedResult;

    export interface IContactsService {
        getContacts: (criteria: IPagedCriteria, callback: (result: IPagedResult<IContactInfo>) => void) => void;
        getContact: (id:number, callback: (result: IContact) => void) => void;
        deleteContact: (contact: IContactInfo, callback: (result: boolean) => void) => void;
        saveContact: (contact: IContact, callback: (result: IContact) => void) => void;
    }

    class ContactsService implements IContactsService {
        getContacts: (criteria: IPagedCriteria, callback: (result: IPagedResult<IContactInfo>) => void) => void;
        getContact: (id: number, callback: (result: IContact) => void) => void;
        deleteContact: (contact: IContactInfo, callback: (result: boolean) => void) => void;
        saveContact: (contact: IContact, callback: (result: IContact) => void) => void;

        constructor(http: app.core.services.http.IHttp, utilities: app.core.services.IUtilities) {

            this.getContacts = function (criteria: IPagedCriteria, callback: (result: IPagedResult<IContactInfo>) => void) {
                return http.get("/Contacts/Get", (pagedResult: IHttpResult<IPagedResult<IContactInfo>>) => {
                    if (pagedResult.Success) {
                        callback(pagedResult.Result);
                    }
                }, true, criteria);
            };

            this.deleteContact = function (contact: IContactInfo, callback: (result: boolean) => void) {
                return http.post("/Contacts/Delete", contact, (result: IHttpResult<boolean>) => {
                    callback(result.Success);
                }, true);
            };

            this.getContact = function (id: number, callback: (result: IContact) => void) {
                return http.get("/Contacts/GetById", (result: IHttpResult<IContact>) => {
                    if (result.Success) {
                        callback(result.Result);
                    }
                }, true, { contactId: id });
            };

            this.saveContact = function (contact: IContact, callback: (result: IContact) => void) {
                return http.post("/Contacts/Save", contact, (result: IHttpResult<IContact>) => {
                    if (result.Result) {
                        callback(result.Result);
                    }
                }, true);
            };
        }
    }

    angular.module("app.contacts.services", [])
        .factory("contactsService", ["http", "utilities", "globalsService",
            function (http: app.core.services.http.IHttp, utilities: app.core.services.IUtilities) {
                return new ContactsService(http, utilities);
            }]);
}